#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* reads in a list of ints from stdin and returns in nums array
 * this is not really trivial in C */
void read_line(int* nums, int* size) {
    /* read the input line as a string */
    char* line = NULL;
    unsigned long length = 0;
    getline(&line, &length, stdin);

    /* set size to 0 */
    *size = 0;

    while (line) {
        /* read next int out of string */
        if (sscanf(line, "%d", &nums[*size]) != 1) {
            break;
        }

        /* increment size */
        *size += 1;

        /* move string to next space */
        line = strstr(line, " ");

        /* and one char past */
        if (line) line++;
    }
}

/* returns the preference matrix from the input */
int** get_matrix(int n) {
    int i, j;

    /* create an n by n adjacency matrix */
    int** matrix = malloc(n * sizeof(int*));
    for (i = 0; i < n; i++) {
        matrix[i] = malloc(n * sizeof(int));
    }

    /* zero out the matrix */
    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            matrix[i][j] = 0;
        }
    }

    /* read in the preferences for the pilots */
    int gunners[n], ngunners;
    for (i = 0; i < n; i++) {
        read_line(gunners, &ngunners);

        /* go through the list and put -1's in this association */
        for (j = 0; j < ngunners; j++) {
            matrix[i][gunners[j] - 1] = -1;
        }
    }

    /* read in the preferences for the gunners */
    int pilots[n], npilots;
    for (i = 0; i < n; i++) {
        read_line(pilots, &npilots);

        /* go through the list and invert the association, -1 turns to 1, 0 remains */
        for (j = 0; j < npilots; j++) {
            matrix[pilots[j] - 1][i] *= -1;
        }
    }

    /* clear out all the -1's which are not reciprocated */
    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            if (matrix[i][j] == -1) {
                matrix[i][j] = 0;
            }
        }
    }

    return matrix;
}

/* counts the number of preferred pairs there are in a pairing */
int happy(int n, int** matrix, int* pairing) {
    int i, total = 0;

    for (i = 0; i < n; i++) {
        int pilot = i;
        int gunner = pairing[i];

        if (matrix[pilot][gunner]) {
            total++;
        } 
    }

    return total;
}

int main() {
    int n, i, j, k;

    /* read the number of pilots/gunners */
    scanf("%d\n", &n);


    /* matrix[pilot][gunner] tells us if they both want to work together */
    int** matrix = get_matrix(n);

    /* this array says which gunner is with pilot i in the current
     * configuration */
    int* pairings = malloc(n * sizeof(int));


    /* initially we set them all to the same i.e. 0-0, 1-1, 2-2, etc. */
    for (i = 0; i < n; i++) {
        pairings[i] = i;
    }

    /* we also have a temporary one for trying new things */
    int* temp_pairings = malloc(n * sizeof(int));


    /* now we make multiple passes where we try to improve this pairing
     * this is an iterative greedy algorithm */
again:
    /* loop through each cell of the matrix */
    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            /* if this cell is on (these guys work well) */
            if (matrix[i][j]) {
                /* copy this pairing */
                for (k = 0; k < n; k++) {
                    temp_pairings[k] = pairings[k];
                }

                /* tentatively assign this pair in the temporary pairing */
                int temp = temp_pairings[j];
                temp_pairings[j] = i;

                /* find who had this gunner and flip it (so the pairing is
                 * still a legit one) */
                for (k = 0; k < n; k++) {
                    if (temp_pairings[k] == i) {
                        temp_pairings[k] = temp;
                    }
                }

                /* count the number of preferred pairings */
                int now = happy(n, matrix, temp_pairings);
                int then = happy(n, matrix, pairings);

                /* if it's more, keep it and go again */
                if (now > then) {
                    for (k = 0; k < n; k++) {
                        pairings[k] = temp_pairings[k];
                    }

                    goto again;
                }
            }
        }
    }

    /* after this it should be optimal */
    int result = happy(n, matrix, temp_pairings);
    printf("The largest number of optimal pairings is %d.\n", result);

    /* clear up memory */
    for (i = 0; i < n; i++) {
        free(matrix[i]);
    }
    free(matrix);
    free(pairings);

    return 0;
}

